<?php

namespace CreoCode\ContaoColumns\Classes;

use Contao\DataContainer;
use Contao\LayoutModel;
use Contao\PageModel;
use Contao\PageRegular;

class ColumnHooks extends \Backend {


	/**
	 * Generate the grix icon in the article list view
	 */
	public function addColumnIcon()
	{
		array_insert($GLOBALS['TL_DCA']['tl_article']['list']['operations'], 0, array
		(
			'columns' => array
			(
				'label'               => &$GLOBALS['TL_LANG']['tl_article']['columns'],
				'icon'                => 'bundles/contaocolumns/be/icons/columns.svg',
				'button_callback'     => array('CreoCode\ContaoColumns\Classes\ColumnHooks', 'createColumnIcon')
			)
		));
	}


	/**
	 * Create the html for the icon
	 */
	public function createColumnIcon($row, $href, $label, $title, $icon, $attributes)
	{
		$icon = 'bundles/contaocolumns/be/icons/columns.svg';
		return '<a href="contao/main.php?do=article_columns&table=tl_content&id='.$row['id'].'&rt='.REQUEST_TOKEN.'" title="'.specialchars($title).'"'.$attributes.'>'
		.\Image::getHtml($icon, $label)
		.'</a> ';
	}


	/**
	 * Add a class to the body when editing an article with grix
	 */
	public function myParseBackendTemplate($strBuffer, $strTemplate)
	{
	    if ($strTemplate == 'be_main' && \Input::get('do') == 'article_columns')
	    {
	        // $strBuffer = str_replace('<body id="top" class="', '<body id="top" class="grix_active grix_lg ', $strBuffer);
	    }
	    return $strBuffer;          
	}


	/**
	 * Generate the grix icon in the article list view
	 */
	public function parseBackendTemplate(string $buffer, string $template)
	{
		if(\Input::get('do') !== 'article_columns') {
			return $buffer;
		}

		if($template == 'be_main'  && \Input::get('do') == 'article_columns') {
			// 1. Create document 
			$domDocument = new \DOMDocument();

			// 2. Handle errors internally
			libxml_use_internal_errors(true);

			// 3. Load your HTML 5
			$domDocument->loadHTML($buffer);

			// 4. Manipulate html
			$xpath = new \DOMXPath($domDocument);
			foreach($xpath->query("//main//div[contains(@class, 'tl_listing')]//div[contains(@class, 'tl_content')]") as $item){
			    $_class = $item->getAttribute('class');
			    // replace indent class with nothing
			    $_class = preg_replace("/(?<=^|\s)inden\w+/", "" , $_class); 
			    // replace odd and even
			    $_class = str_replace(['odd', 'even'], '', $_class);
			    $item->setAttribute('class', $_class);
			}

			// Wrap with new div
			$new_div = $domDocument->createElement('div');
			$new_div->setAttribute('class','tl_columns_backend');
			foreach ($xpath->query("//main//div[contains(@class, 'tl_listing')]//ul") as $node) {
			    //Clone our created div
			    $new_div_clone = $new_div->cloneNode();
			    //Replace image with this wrapper div
			    $node->parentNode->replaceChild($new_div_clone,$node);
			    //Append this image to wrapper div
			    $new_div_clone->appendChild($node);
			}

			// remove <ul ..> but leave childs
			foreach ($xpath->query("//main//div[contains(@class, 'tl_listing')]//ul") as $node) {
			   $parent = $node->parentNode;
			   while ($node->hasChildNodes()) {
			       $parent->insertBefore($node->lastChild, $node->nextSibling);
			   }
			   $parent->removeChild($node);
			}

			// remove <li ..> but leave childs
			foreach ($xpath->query("//main//div[contains(@class, 'tl_listing')]//li") as $node) {
			    $parent = $node->parentNode;
			    while ($node->hasChildNodes()) {
			        $parent->insertBefore($node->lastChild, $node->nextSibling);
			    }
			    $parent->removeChild($node);
			}

			// Remove drag handlers
			foreach ($xpath->query("//main//div[contains(@class, 'tl_listing')]//*[contains(@class, 'drag-handle')]") as $node) {
			   $parent = $node->parentNode;
			   $parent->removeChild($node);
			}


			// Columns manipulator
			$_columns = [];
			foreach ($xpath->query("//main//div[contains(@class, 'tl_listing')]//div[contains(@class, 'tl_content')][contains(@class, 'wrapper_')]//div[contains(@class, 'column')]/../..") as $node) {
				$_h40 = $xpath->query("./div[contains(@class, 'h40')]", $node)->item(0);
				$_column = $xpath->query("./div", $_h40)->item(0);
				$_tlContentRight = $xpath->query("./div[contains(@class, 'tl_content_right')]", $node)->item(0);

				if($_column) {
					$_id = $_column->getAttribute('data-id');
					$_type = $_column->getAttribute('data-type');

					$link = $domDocument->createElement('div');
					$link->setAttribute('data-id', $_id);
					$link->setAttribute('data-type', $_type);

					$el = $node;
					$el->parentNode->insertBefore($link, $el);
					$el->parentNode->removeChild($el);

					$_columns[$_id] = [
						'type' => $_type,
						'tl_content_right' => $domDocument->saveHTML($_tlContentRight),
					];
				}
			}

			$buffer = $domDocument->saveHTML();

			// Replace here.. domdocument auto closes tags
			$GLOBALS['_tmp_columns_backend_view'] = true;
			foreach ($_columns as $key => $value) {
				$_template = \Controller::getContentElement($key);
				$_template = str_replace('<div class="tl_content_right"></div>', $value['tl_content_right'], $_template);
				$buffer = str_replace('<div data-id="'.$key.'" data-type="'.$value['type'].'"></div>', $_template, $buffer);
			}
			$GLOBALS['_tmp_columns_backend_view'] = false;

			// 5. Clear errors
			$errors = libxml_get_errors();

			// 6. If you need to know about the errors or warnings
			foreach ($errors as $error)
			{
			    /* @var $error LibXMLError */
			    /*
			    each $error variable contains a LibXMLError object with the following properties
			    array(
			        'level' => 2,
			        'code' => 801,
			        'column' => 28,
			        'message' => 'Tag figcaption invalid',
			        'file' => '',
			        'line' => 10,
			    )
			    */
			}
		}

		return $buffer;
	}

}
